
/******************************************************************************
* MODULE     : real_shift.cpp
* DESCRIPTION: Shifting a real number
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class shift_real_rep: public real_rep {
private:
  real x;
  mp_exp_t shift;
public:
  inline shift_real_rep (real x2, mp_exp_t shift2):
    real_rep (weight (x2)), x (x2), shift (shift2) {}
  void print (ostream& out) {
    if (shift>=0) out << "(<< " << x << " " << shift << ")";
    else out << "(>> " << x << " " << (-shift) << ")"; }
  dyadic get_bound () { return bound (x); }
  interval approx (const dyadic& err);
};

interval
shift_real_rep::approx (const dyadic& err) {
  if (wider (best, err)) {
    dyadic err_x= spread (err >> shift);
    interval xa = x->approx (err_x);
    mp_prec_t new_prec= linear_algo (err);
    best= reduce (xa, new_prec) << shift;
    if (wider (best, err))
      fatal_error ("Bad a priori error estimation",
		   "shift_real_rep::approx", "real_shift.cpp");
  }
  return best;
}

real
operator << (const real& x, mp_exp_t exp) {
  return new shift_real_rep (x, exp);
}

real
operator >> (const real& x, mp_exp_t exp) {
  return new shift_real_rep (x, -exp);
}

}
